@@ -19,14 +19,16 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
+#include <asm/mach-types.h>
+
 #include <linux/delay.h>
 #include <linux/gpio.h>
 #include <linux/regulator/consumer.h>
 #include <linux/resource.h>
-#include <asm/mach-types.h>
 #include <linux/platform_device.h>
 #include <linux/pwm_backlight.h>
 #include <linux/err.h>
+
 #include <mach/nvhost.h>
 #include <mach/nvmap.h>
 #include <mach/irqs.h>
@@ -248,8 +250,8 @@ static struct resource paz00_disp1_resources[] = {
 	},
 	{
 		.name	= "fbmem",
-		.start	= 0x1c012000,
-		.end	= 0x1c012000 + 0x258000 - 1,   /* 2.4 MB @ 448 MB */
+		.start	= 0x1fd95000,
+		.end	= 0x1fd95000 + 0x26B000 - 1,   // 2.4 MB @ 509 MB
 		.flags	= IORESOURCE_MEM,
 	},
 };
@@ -267,12 +269,6 @@ static struct resource paz00_disp2_resources[] = {
 		.end	= TEGRA_DISPLAY2_BASE + TEGRA_DISPLAY2_SIZE-1,
 		.flags	= IORESOURCE_MEM,
 	},
-	{
-		.name	= "fbmem",
-		.start	= 0x1c26A000,
-		.end	= 0x1c26A000 + 0x500000 - 1,  /* 5 MB */
-		.flags	= IORESOURCE_MEM,
-	},
 	{
 		.name	= "hdmi_regs",
 		.start	= TEGRA_HDMI_BASE,
@@ -306,8 +302,8 @@ static struct tegra_fb_data paz00_fb_data = {
 
 static struct tegra_fb_data paz00_hdmi_fb_data = {
 	.win		= 0,
-	.xres		= 1024,
-	.yres		= 600,
+	.xres		= 1280,
+	.yres		= 720,
 	.bits_per_pixel	= 16,
 };
 
@@ -338,6 +334,9 @@ static struct tegra_dc_out paz00_disp2_out = {
 
 	.enable		= paz00_hdmi_enable,
 	.disable	= paz00_hdmi_disable,
+
+	/* DVFS tables only updated up to 148.5MHz for HDMI currently */
+	.max_pclk_khz   = 148500,
 };
 
 static struct tegra_dc_platform_data paz00_disp1_pdata = {
@@ -415,7 +414,7 @@ static struct platform_device *paz00_gfx_devices[] __initdata = {
 	&paz00_backlight_device,
 };
 
-int __init paz00_panel_init(void)
+int __init paz00_panel_init(size_t fb_addr)
 {
 	int err;
 
@@ -434,6 +433,8 @@ int __init paz00_panel_init(void)
 	tegra_gpio_enable(TEGRA_HDMI_HPD);
 	gpio_free(TEGRA_HDMI_HPD);
 
+	paz00_disp1_resources[2].start = (unsigned long)fb_addr;
+
 	err = platform_add_devices(paz00_gfx_devices,
 				   ARRAY_SIZE(paz00_gfx_devices));
 
