@@ -29,7 +29,7 @@
 #include "board-paz00.h"
 
 #define PMC_CTRL                0x0
-#define PMC_CTRL_INTR_LOW       (1<<7)
+#define PMC_CTRL_INTR_LOW       (1<<17)
 
 static struct regulator_consumer_supply tps658621_sm0_supply[] = {
 	REGULATOR_SUPPLY("vdd_core", NULL),
@@ -55,6 +55,7 @@ static struct regulator_consumer_supply tps658621_ldo1_supply[] = { /* 1V2 */
 };
 static struct regulator_consumer_supply tps658621_ldo2_supply[] = { /* VDD_RTC */
 	REGULATOR_SUPPLY("vdd_rtc", NULL),
+	REGULATOR_SUPPLY("vdd_aon", NULL),
 };
 static struct regulator_consumer_supply tps658621_ldo3_supply[] = { /* 3V3 */
 	REGULATOR_SUPPLY("avdd_usb", NULL),
@@ -125,12 +126,12 @@ static struct regulator_consumer_supply tps658621_soc_supply[] = {
 		.consumer_supplies = tps658621_##_id##_supply,		\
 	}
 
-static struct regulator_init_data sm0_data  = REGULATOR_INIT(sm0,  725,  1200, true);
-static struct regulator_init_data sm1_data  = REGULATOR_INIT(sm1,  725,  1200, true);
+static struct regulator_init_data sm0_data  = REGULATOR_INIT(sm0,  725,  1300, true);
+static struct regulator_init_data sm1_data  = REGULATOR_INIT(sm1,  725,  1125, true);
 static struct regulator_init_data sm2_data  = REGULATOR_INIT(sm2,  3000, 3700, true);
 static struct regulator_init_data ldo0_data = REGULATOR_INIT(ldo0, 1250, 3300, false);
 static struct regulator_init_data ldo1_data = REGULATOR_INIT(ldo1, 725,  1100, true);
-static struct regulator_init_data ldo2_data = REGULATOR_INIT(ldo2, 725,  1200, false);
+static struct regulator_init_data ldo2_data = REGULATOR_INIT(ldo2, 725,  1275, false);
 static struct regulator_init_data ldo3_data = REGULATOR_INIT(ldo3, 1250, 3300, false);
 static struct regulator_init_data ldo4_data = REGULATOR_INIT(ldo4, 1700, 1800, true);
 static struct regulator_init_data ldo5_data = REGULATOR_INIT(ldo5, 1250, 2850, true);
@@ -152,16 +153,7 @@ static struct regulator_init_data buck_data = REGULATOR_INIT(buck, 1250, 3300);
 
 /* FIXME: do we have rtc alarm irq? */
 static struct tps6586x_rtc_platform_data paz00_rtc_data = {
-/*	.irq	= TEGRA_NR_IRQS + TPS6586X_INT_RTC_ALM1, */
-	.irq	= -1,
-/*	.start	= {
-			.year	= 2009,
-			.month	= 1,
-			.day	= 1,
-			.hour	= 0,
-			.min	= 0,
-			.sec	= 0,
-		}, */
+	.irq	= TEGRA_NR_IRQS + TPS6586X_INT_RTC_ALM1,
 };
 
 static struct tps6586x_subdev_info tps_devs[] = {
@@ -191,11 +183,13 @@ static struct tps6586x_platform_data tps_platform = {
 	.num_subdevs	= ARRAY_SIZE(tps_devs),
 	.subdevs	= tps_devs,
 	.gpio_base	= TEGRA_NR_GPIOS,
+	.irq_base	= TEGRA_NR_IRQS,
 };
 
 static struct i2c_board_info __initdata paz00_regulators[] = {
 	{
 		I2C_BOARD_INFO("tps6586x", 0x34),
+		.irq	= INT_EXTERNAL_PMU,
 		.platform_data = &tps_platform,
 	},
 };
