@@ -348,6 +348,11 @@ static inline bool phy_is_ulpi(struct tegra_usb_phy *phy)
 	return (phy->instance == 1);
 }
 
+static inline bool phy_is_utmi(struct tegra_usb_phy *phy)
+{
+	return (phy->instance != 1);
+}
+
 static int utmip_pad_open(struct tegra_usb_phy *phy)
 {
 	phy->pad_clk = clk_get_sys("utmip-pad", NULL);
@@ -423,6 +428,44 @@ static int utmip_pad_power_off(struct tegra_usb_phy *phy, bool is_dpd)
 	return 0;
 }
 
+void utmi_phy_vbus_on(struct tegra_usb_phy *phy)
+{
+	struct tegra_utmip_config *config = phy->config;
+
+	if (config->vbus_gpio) {
+		/*
+		 * For those platforms vbus_en and oc are shared, we
+		 * need to change that gpio back to input signal
+		 * for handling the over current event.
+		 */
+		if (config->shared_pin_vbus_en_oc)
+			gpio_direction_input(config->vbus_gpio);
+		else
+			gpio_set_value(config->vbus_gpio, 1);
+	}
+}
+
+void utmi_phy_vbus_off(struct tegra_usb_phy *phy)
+{
+	struct tegra_utmip_config *config = phy->config;
+
+	if (config->vbus_gpio) {
+		/*
+		 * For those platforms vbus_en and oc are shared, the default
+		 * state of the signal should be logic '1' meaning it works
+		 * as host port and the over current event should be active
+		 * low. In order to turn off vbus in suspend and also support
+		 * over current event in USB working state, that gpio direction
+		 * should be programmed as input signal except when turning off
+		 * vbus in suspend.
+		 */
+		if (config->shared_pin_vbus_en_oc)
+			gpio_direction_output(config->vbus_gpio, 0);
+		else
+			gpio_set_value(config->vbus_gpio, 0);
+	}
+}
+
 static int utmi_wait_register(void __iomem *reg, u32 mask, u32 result)
 {
 	unsigned long timeout = 2000;
@@ -771,14 +814,14 @@ static int ulpi_phy_power_on(struct tegra_usb_phy *phy, bool is_dpd)
 	 * "disconnected" and cause trouble on later resume.
 	 */
 	if (!phy->ulpi_initialized) {
+		clk_enable(phy->clk);
 		gpio_direction_output(config->reset_gpio, 0);
 		udelay(1);
 		gpio_direction_output(config->reset_gpio, 1);
+		/* Wait Tstart max. 2.37 ms after RESET# is deasserted */
+		usleep_range(2370, 3000);
 	}
 
-	clk_enable(phy->clk);
-	msleep(1);
-
 	val = readl(base + USB_SUSP_CTRL);
 	val |= UHSIC_RESET;
 	writel(val, base + USB_SUSP_CTRL);
@@ -832,7 +875,11 @@ static int ulpi_phy_power_on(struct tegra_usb_phy *phy, bool is_dpd)
 	val = readl(base + USB_SUSP_CTRL);
 	val &= ~USB_SUSP_CLR;
 	writel(val, base + USB_SUSP_CTRL);
-	udelay(1);
+
+	/* Wait for ULPI PHY clock to be valid */
+	if (utmi_wait_register(base + USB_SUSP_CTRL, USB_PHY_CLK_VALID,
+	     USB_PHY_CLK_VALID))
+		pr_err("%s: timeout waiting for phy to stabilize\n", __func__);
 
 	val = readl(base + USB_PORTSC1);
 	val |= USB_PORTSC1_WKOC | USB_PORTSC1_WKDS | USB_PORTSC1_WKCN;
@@ -846,18 +893,23 @@ static void ulpi_phy_power_off(struct tegra_usb_phy *phy, bool is_dpd)
 	void __iomem *base = phy->regs;
 	struct tegra_ulpi_config *config = phy->config;
 
-	/* Clear WKCN/WKDS/WKOC wake-on events that can cause the USB
+	/* Clear WKDS/WKOC wake-on events that can cause the USB
 	 * Controller to immediately bring the ULPI PHY out of low power.
+	 * Don't clear USB_PORTSC1_WKCN, it may be set by upper layer driver.
+	 * We need it to wake up PHY from suspend state if device is connected.
+	 * So .bus_resume function in ehci-tegra.c can be called to resume bus.
 	 * Set PHCD bit for PHY to enter low power mode
 	 * to keep current line status as "connected" on D+, D-
 	 * instead of pulling the reset pin of PHY chip to save power.
 	 */
 	val = readl(base + USB_PORTSC1);
-	val &= ~(USB_PORTSC1_WKOC | USB_PORTSC1_WKDS | USB_PORTSC1_WKCN);
+	val &= ~(USB_PORTSC1_WKOC | USB_PORTSC1_WKDS);
 	val |= USB_PORTSC1_PHCD;
 	writel(val, base + USB_PORTSC1);
 
-	clk_disable(phy->clk);
+	/* Wait for ULPI PHY to be suspended */
+	if (utmi_wait_register(base + USB_SUSP_CTRL, USB_PHY_CLK_VALID, 0) < 0)
+		pr_err("%s: timeout waiting for phy to stabilize\n", __func__);
 }
 
 static int null_phy_power_on(struct tegra_usb_phy *phy, bool is_dpd)
@@ -1252,6 +1304,50 @@ void tegra_usb_phy_power_off(struct tegra_usb_phy *phy, bool is_dpd)
 }
 EXPORT_SYMBOL_GPL(tegra_usb_phy_power_off);
 
+void tegra_usb_phy_utmi_vbus_init(struct tegra_utmip_config *utmi_config,
+					const char *label)
+{
+	int gpio_status;
+
+	gpio_status = gpio_request(utmi_config->vbus_gpio, label);
+	if (gpio_status < 0) {
+		pr_err("%s request GPIO FAILED\n", label);
+		goto vbus_gpio_init_exit;
+	}
+
+	if (utmi_config->shared_pin_vbus_en_oc)
+		gpio_status = gpio_direction_input(utmi_config->vbus_gpio);
+	else
+		gpio_status = gpio_direction_output(utmi_config->vbus_gpio, 1);
+
+	if (gpio_status < 0) {
+		pr_err("%s request GPIO DIRECTION FAILED\n", label);
+		gpio_free(utmi_config->vbus_gpio);
+		goto vbus_gpio_init_exit;
+	}
+
+	if (!utmi_config->shared_pin_vbus_en_oc)
+		gpio_set_value(utmi_config->vbus_gpio, 1);
+
+vbus_gpio_init_exit:
+	if (gpio_status < 0) {
+		WARN_ON(1);
+		utmi_config->vbus_gpio = 0;
+	}
+}
+
+void tegra_usb_phy_vbus_on(struct tegra_usb_phy *phy)
+{
+	if (phy_is_utmi(phy))
+		utmi_phy_vbus_on(phy);
+}
+
+void tegra_usb_phy_vbus_off(struct tegra_usb_phy *phy)
+{
+	if (phy_is_utmi(phy))
+		utmi_phy_vbus_off(phy);
+}
+
 void tegra_usb_phy_preresume(struct tegra_usb_phy *phy, bool is_dpd)
 {
 	if (!phy_is_ulpi(phy))
@@ -1299,9 +1395,12 @@ void tegra_usb_phy_close(struct tegra_usb_phy *phy)
 	if (phy_is_ulpi(phy)) {
 		struct tegra_ulpi_config *ulpi_config = phy->config;
 
-		if (ulpi_config->inf_type == TEGRA_USB_LINK_ULPI)
+		if (ulpi_config->inf_type == TEGRA_USB_LINK_ULPI) {
+			clk_disable(phy->clk);
 			clk_put(phy->clk);
-	} else
+		}
+	}
+	else
 		utmip_pad_close(phy);
 	if (phy->mode == TEGRA_USB_PHY_MODE_HOST) {
 		vbus_disable(usb_phy_data[phy->instance].vbus_gpio);
