@@ -31,6 +31,7 @@
 #define TEGRA_USB_PHY_CLK_VALID			(1 << 7)
 #define TEGRA_USB_SRT				(1 << 25)
 #define TEGRA_USB_PHY_CLK_VALID_INT_ENB        (1 << 9)
+#define TEGRA_USB_PHY_CLK_VALID_INT_STS        (1 << 8)
 
 #define TEGRA_USB_PORTSC1_OFFSET		0x184
 #define TEGRA_USB_PORTSC1_WKCN			(1 << 20)
@@ -41,6 +42,8 @@
 #define TEGRA_USB_DMA_ALIGN 32
 
 #define STS_SRI	(1<<7)	/*	SOF Recieved	*/
+#define PORT_SPEED_MASK		(3 << 26)
+#define PORT_SPEED_RESERVED	(3 << 26)
 
 struct tegra_ehci_hcd {
 	struct ehci_hcd *ehci;
@@ -53,6 +56,7 @@ struct tegra_ehci_hcd {
 	int bus_suspended;
 	int port_resuming;
 	int power_down_on_bus_suspend;
+	int keep_clock_in_bus_suspend;
 	struct delayed_work work;
 	enum tegra_usb_phy_port_speed port_speed;
 };
@@ -62,7 +66,8 @@ static void tegra_ehci_power_up(struct usb_hcd *hcd, bool is_dpd)
 	struct tegra_ehci_hcd *tegra = dev_get_drvdata(hcd->self.controller);
 
 	clk_enable(tegra->emc_clk);
-	clk_enable(tegra->sclk_clk);
+	if (!tegra->keep_clock_in_bus_suspend)
+		clk_enable(tegra->sclk_clk);
 	tegra_usb_phy_power_on(tegra->phy, is_dpd);
 	tegra->host_resumed = 1;
 }
@@ -73,26 +78,34 @@ static void tegra_ehci_power_down(struct usb_hcd *hcd, bool is_dpd)
 
 	tegra->host_resumed = 0;
 	tegra_usb_phy_power_off(tegra->phy, is_dpd);
+	if (!tegra->keep_clock_in_bus_suspend)
+		clk_disable(tegra->sclk_clk);
 	clk_disable(tegra->emc_clk);
-	clk_disable(tegra->sclk_clk);
 }
 
 static irqreturn_t tegra_ehci_irq (struct usb_hcd *hcd)
 {
 	struct ehci_hcd *ehci = hcd_to_ehci (hcd);
+	struct ehci_regs __iomem *hw = ehci->regs;
 	u32 val;
 
 	spin_lock (&ehci->lock);
 	val = readl(hcd->regs + TEGRA_USB_SUSP_CTRL_OFFSET);
-	val &= ~TEGRA_USB_PHY_CLK_VALID_INT_ENB;
-	writel(val , (hcd->regs + TEGRA_USB_SUSP_CTRL_OFFSET));
+	if ((val  & TEGRA_USB_PHY_CLK_VALID_INT_STS)) {
+		val &= ~TEGRA_USB_PHY_CLK_VALID_INT_ENB | TEGRA_USB_PHY_CLK_VALID_INT_STS;
+		writel(val , (hcd->regs + TEGRA_USB_SUSP_CTRL_OFFSET));
 
-	val = readl(hcd->regs + TEGRA_USB_PORTSC1_OFFSET);
-	val &= ~TEGRA_USB_PORTSC1_WKCN;
-	writel(val , (hcd->regs + TEGRA_USB_PORTSC1_OFFSET));
+		val = readl(hcd->regs + TEGRA_USB_PORTSC1_OFFSET);
+		val &= ~TEGRA_USB_PORTSC1_WKCN;
+		writel(val , (hcd->regs + TEGRA_USB_PORTSC1_OFFSET));
 
+		val = readl(&hw->status);
+		if (!(val  & STS_PCD)) {
+			spin_unlock (&ehci->lock);
+			return 0;
+		}
+	}
 	spin_unlock (&ehci->lock);
-
 	return ehci_irq(hcd);
 }
 
@@ -111,7 +124,7 @@ static int tegra_ehci_hub_control(
 	u32 __iomem	*status_reg;
 	u32		usbsts_reg;
 	unsigned long	flags;
-	int		retval = 0;
+	int		i, retval = 0;
 	unsigned	selector;
 	struct		tegra_ehci_hcd *tegra = dev_get_drvdata(hcd->self.controller);
 	bool		hsic = false;
@@ -144,6 +157,43 @@ static int tegra_ehci_hub_control(
 			/* Resume completed, re-enable disconnect detection */
 			tegra->port_resuming = 0;
 			tegra_usb_phy_postresume(tegra->phy, false);
+			/*
+			 * After tegra_usb_phy_postresume is called, device
+			 * may temporarily "disconnect", setting PORTSC bit 1.
+			 * Driver hub.c would then think the device is removed
+			 * from USB bus and "logically disconnect" the device.
+			 * On subsequent reconnect, a Bootable USB disk will be
+			 * enumerated with new device address, making the
+			 * original device address unaccessible. To avoid this,
+			 * in case the device is still connected, wait 120 ms
+			 * for bit 0 of PORTSC to be set. Bit 0 signals the
+			 * PORTSC connection state is stable.
+			 *
+			 * Page 150, Figure 7-29 of USB2.0 spec:
+			 *   t1 = 20 ms = bPwr2PwrGood of root hub descriptor
+			 *     If VBUS is turned off in system suspend state.
+			 *   t2 is defined in Section 7.1.7.3 of USB 2.0 spec:
+			 *     "t2 must be less than 100 ms for all hub and
+			 *      device implementation".
+			 * t1+t2 (20+100 ms) is timeout used in loop below.
+			 *
+			 * If your USB disk doesn't comply with USB 2.0 spec,
+			 * please try increasing timeout from 120 to 1000 (or
+			 * more) and report results to linux-usb mailing list.
+			 */
+			if ((temp & PORT_SPEED_MASK) !=
+				 PORT_SPEED_RESERVED ||
+				(temp & PORT_CSC)) {
+				for (i = 0; i < 120; i++) {
+					spin_unlock_irqrestore(&ehci->lock,
+						flags);
+					usleep_range(1000, 1001);
+					spin_lock_irqsave(&ehci->lock, flags);
+					temp = ehci_readl(ehci, status_reg);
+					if (temp & PORT_CONNECT)
+						break;
+				}
+			}
 		}
 	} else if (typeReq == SetPortFeature && wValue == USB_PORT_FEAT_SUSPEND) {
 		temp = ehci_readl(ehci, status_reg);
@@ -917,6 +967,7 @@ static int tegra_ehci_probe(struct platform_device *pdev)
 
 	tegra->host_resumed = 1;
 	tegra->power_down_on_bus_suspend = pdata->power_down_on_bus_suspend;
+	tegra->keep_clock_in_bus_suspend = pdata->keep_clock_in_bus_suspend;
 	tegra->ehci = hcd_to_ehci(hcd);
 
 	irq = platform_get_irq(pdev, 0);
@@ -987,8 +1038,10 @@ static int tegra_ehci_resume(struct platform_device *pdev)
 	struct tegra_ehci_hcd *tegra = platform_get_drvdata(pdev);
 	struct usb_hcd *hcd = ehci_to_hcd(tegra->ehci);
 
-	if ((tegra->bus_suspended) && (tegra->power_down_on_bus_suspend))
+	if ((tegra->bus_suspended) && (tegra->power_down_on_bus_suspend)) {
+		tegra_usb_phy_vbus_on(tegra->phy);
 		return 0;
+	}
 
 	return tegra_usb_resume(hcd, true);
 }
@@ -998,8 +1051,15 @@ static int tegra_ehci_suspend(struct platform_device *pdev, pm_message_t state)
 	struct tegra_ehci_hcd *tegra = platform_get_drvdata(pdev);
 	struct usb_hcd *hcd = ehci_to_hcd(tegra->ehci);
 
-	if ((tegra->bus_suspended) && (tegra->power_down_on_bus_suspend))
+	if ((tegra->bus_suspended) && (tegra->power_down_on_bus_suspend)) {
+		/*
+		 * We turn off vbus for those platforms which are really care
+		 * about USB power consumption, i.e. for those have the flag
+		 * power_down_on_bus_suspend set.
+		 */
+		tegra_usb_phy_vbus_off(tegra->phy);
 		return 0;
+	}
 
 	if (time_before(jiffies, tegra->ehci->next_statechange))
 		msleep(10);
